// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 25th May 2006
//
// Purpose:       Demo shows how to use call-stack-trace feature
//
// Notes:
//                See the makefile for neccessary compiler flags.
//                BTW, please note the call-stack-trace feature
//                makes the program execution very slow, so I recommend
//                to use this feature only when you really need a callstack.
//
// -----------------------------------------------------------------------------
#include <hel2.h>


// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------

void ATTR_NOINLINE FirstFunction(void);
void ATTR_NOINLINE SecondFunction(void);
void ATTR_NOINLINE ThirdFunction(void);
void ATTR_NOINLINE FourthFunction(const void* pNoNullPointer);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);

    FirstFunction();

    return 0;
}

// -----------------------------------------------------------------------------
// FirstFunction
// -----------------------------------------------------------------------------
void ATTR_NOINLINE FirstFunction(void)
{
  SecondFunction();
}

// -----------------------------------------------------------------------------
// SecondFunction
// -----------------------------------------------------------------------------
void ATTR_NOINLINE SecondFunction(void)
{
  ThirdFunction();
}

// -----------------------------------------------------------------------------
// ThirdFunction
// -----------------------------------------------------------------------------
void ATTR_NOINLINE ThirdFunction(void)
{
  FourthFunction(NULL);
}

// -----------------------------------------------------------------------------
// FourthFunction
// -----------------------------------------------------------------------------
void ATTR_NOINLINE FourthFunction(const void* pNoNullPointer)
{
  HEL_ASSERT(pNoNullPointer != NULL, "pNoNullPointer must not be null.");
}

