// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 18th April 2006
//
// Purpose:       Demo shows how to use the window-system,
//                to be more precise, it uses the object-window.
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"
#include "lookups.h"


// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------
#define NUM_SPRITES 40


// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
THandle ATTR_EWRAM g_Sprites[NUM_SPRITES];
volatile u32 ATTR_EWRAM g_FramesTotal=0;

// Allocate memory for Object-System
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_ObjSystemBuffer[HEL_SUBSYSTEM_OBJ_REQUIREDMEMORY];

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    u8 i;
    
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    ham_SetBgMode(4);

    // Initialize the Object-System. If you do not call
    // this function, the program behaviour is undefined
    hel_ObjInit(g_ObjSystemBuffer);
    
    // Initialize the Window-System. If you do not call
    // this function, the program behaviour is undefined
    hel_WinInit();
    
    // Load Backgroundpicture and immediatly
    // flip the backbuffer to display its content.
    hel_BmpLoad(ResData(RES_GIRLBG_RAW));
    ham_FlipBGBuffer();

   // Create an object (sprite)
   g_Sprites[0] = hel_ObjCreate(ResData(RES_BALLS_RAW),  // A pointer to the tile data for this object
                              OBJ_SHAPE_SQUARE,          // obj_shape
                              2,                       // obj_size
      /* See here ------> */  OBJ_MODE_OBJWINDOW,      // obj_mode
                              1,                       // col_mode
                              0,                       // pal_no
                              FALSE,                   // mosaic
                              FALSE,                   // hflip
                              FALSE,                   // vflip
                              FALSE,                   // dbl_size
                              0,                       // prio
                              30,                      // x position of sprite on screen
                              30);                     // y position of sprite on screen
                              
    // Clone object
    for(i=1; i<NUM_SPRITES; i++)
    {
        // Cloning a sprite means having another ham_obj entry that you can play
        // with that uses a copy of the data set up for the sprite specified in objno.
    	 g_Sprites[i] = hel_ObjClone(g_Sprites[0], 1, 1);
    }

    // Setup an OBJ-Window
    // Since only transparent obj-pixels form the window,
    // we do not need to specify any region
    hel_WinCreate(2, 0, 0, 0, 0, WIN_NONE, WIN_ALL);

    // Load Background Palette
    hel_PalBgLoad256(ResData(RES_GIRLBG_PAL));
    
    // Inits the Palette for the Sprite
    hel_PalObjLoad256(ResData16(RES_BALLS_PAL));
    
    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);

    for(;;)
    {
        for(i=0;i<NUM_SPRITES;i++)
        {
            // Update X and Y Sprite Coordinates
            hel_ObjSetXY
              (
                g_Sprites[i],                           // Obj Handle
                (wobble_x[(g_FramesTotal+(i*2))%120]),  // X
                (wobble_y[(g_FramesTotal+(i*3))%300])   // Y
              );
        }
        
        hel_SwiVBlankIntrWait();
    }
}


// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
   // Transmit HEL's shadowed Object Attributes
   // to Hardware to see changes.
   hel_ObjTransmit();

   // Transmit HEL's shadowed Window Attributes
   // to Hardware to see changes.
   hel_WinTransmit();

   g_FramesTotal++;
   
   hel_IntrAcknowledge(INT_TYPE_VBL);
}


